@@ -36,15 +36,18 @@ import shelve
 import pango
 import tempfile
 import shutil
+import sqlite3
 import subprocess
 
 from os.path import basename, exists, join, splitext
 
-import database
+import core.database
+import fontutils
 
 from constants import ALIAS_FAMILIES, CACHE_FILE, USER, USER_LIBRARY_DIR, \
                         T1_EXTS, FONT_EXTS
-from utils.common import delete_cache, natural_sort, strip_archive_ext
+from utils.common import delete_cache, delete_database, natural_sort, \
+                            strip_archive_ext
 from utils.xmlutils import get_blacklisted, load_collections
 
 
@@ -71,8 +74,8 @@ class Collection(object):
                 self.families.append(family)
         else:
             raise TypeError\
-            ('Expected a string, tuple or list but got %s instead' % \
-            type(families[0]))
+            ('Expected a string, tuple or list but got {0!s} instead'.format(
+            type(families[0])))
         self.families = list(set(self.families))
         return
 
@@ -80,9 +83,7 @@ class Collection(object):
         """
         Return True if family is already in collection.
         """
-        if family in self.families:
-            return True
-        return False
+        return (family in self.families)
 
     def get_label(self):
         """
@@ -112,8 +113,8 @@ class Collection(object):
                     self.families.remove(family)
         else:
             raise TypeError\
-            ('Expected a string, tuple or list but got %s instead' % \
-            type(families[0]))
+            ('Expected a string, tuple or list but got {0!s} instead'.format(
+            type(families[0])))
         return
 
     def set_comment(self, comment):
@@ -125,13 +126,14 @@ class Collection(object):
         if isinstance(comment, str):
             self.comment = comment
         else:
-            raise TypeError('Expected a string but got %s instead' \
-                                                        % type(comment))
+            raise TypeError('Expected a string but got {0!s} instead'.format(
+                            type(comment)))
+
 
 class Face(object):
     """
     This class holds details about a face.
-    
+
     Provides an equivalent to pango.FontFace that can be pickled.
     """
     __slots__ = ('name', 'description')
@@ -164,26 +166,27 @@ class Family(object):
     """
     This class holds details about a font family.
     """
-    __slots__ = ('name', 'user', 'enabled', 'pango_family', 'styles')
+    __slots__ = ('name', 'user', 'enabled', 'pango_family', 'styles', 'version')
     def __init__(self, family):
         self.name = family
         self.user = False
         self.enabled = True
         self.pango_family = None
         self.styles = {}
+        self.version = 1
 
     def get_count(self, display = True):
         """
         Return the number of styles.
-        
+
         If display is True, return a string suitable for display, i.e '5 Fonts'
         """
         count = len(self.styles)
         if display:
             if count > 1:
-                return _('%s Fonts') % count
+                return _('{0} Fonts').format(count)
             else:
-                return _('%s Font') % count
+                return _('{0} Font').format(count)
         else:
             return count
 
@@ -203,10 +206,25 @@ class Family(object):
         return self.name
 
 
+class FileDetails(object):
+    """
+    This class holds file details about a particular style.
+    """
+    __slots__ = ('filepath', 'filetype', 'filesize', 'psname')
+    def __init__(self, sqlite_row):
+        self.filepath = sqlite_row['filepath']
+        self.filetype = sqlite_row['filetype']
+        self.filesize = sqlite_row['filesize']
+        self.psname = sqlite_row['psname']
+
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+
 class PangoFamily(object):
     """
     This class holds details about a pango family.
-    
+
     Provides an equivalent to pango.FontFamily that can be pickled.
     """
     __slots__ = ('name', 'faces', 'mono')
@@ -268,18 +286,19 @@ class Sort(object):
         else:
             self.widget = gtk.Window()
         self.manager = fontmanager
-        database.sync()
-        self.table = database.Table('Fonts')
-        self.cache = shelve.open(CACHE_FILE, protocol=cPickle.HIGHEST_PROTOCOL)
+        self.cache = None
         self._check_cache()
-        self.file_total = len(self.table)
+        self._check_db()
+        fontutils.set_progress_callback(progress_callback)
+        fontutils.sync_font_database(_('Querying installed files...'))
+        self.table = core.database.Table('Fonts')
         # List of all indexed families
         self.indexed = []
         self._get_indexed()
         # List of actually available families
-        self.all_available = []
+        self.available = []
+        self.pango_families = []
         self._get_available()
-        self.available = [f for f in self.all_available if f in self.indexed]
         # List of system families
         self.system = []
         self._get_system_families()
@@ -304,33 +323,31 @@ class Sort(object):
         if not parent:
             self.widget.destroy()
 
-    def _build_styles_dict(self, family):
+    def _check_db(self):
         """
-        Return a dictionary holding details for all files belonging to family.
+        Delete database if it's in an outdated format.
         """
-        styles = {}
-        for row in self.table.get('*', 'family="%s"' % family):
-            # Too bad sqlite rows don't survive a pickle...
-            keys = row.keys()
-            face = {}
-            for i in range(15):
-                face[keys[i]] = row[i]
-            styles[row['style']] = face
-        return styles
+        try:
+            test = core.database.Table('Fonts')['pstyle']
+        except sqlite3.ProgrammingError:
+            delete_database()
+            self._load_new_cache()
+        test = None
+        return
 
     def _check_cache(self):
         """
         Delete cache if it's in an outdated format.
         """
         try:
-            for key in self.cache.keys():
-                test = self.cache[key]
-                break
-        except cPickle.UnpicklingError:
-            self.cache.close()
-            delete_cache()
             self.cache = shelve.open(CACHE_FILE,
                                         protocol=cPickle.HIGHEST_PROTOCOL)
+            test = self.cache[self.cache.keys()[0]]
+            assert (test.version == 1)
+        except (AssertionError, AttributeError,
+                    IndexError, cPickle.UnpicklingError):
+            self._load_new_cache()
+        test = None
         return
 
     def _disable_rejects(self):
@@ -355,7 +372,9 @@ class Sort(object):
             name = family.get_name()
             if name in ALIAS_FAMILIES:
                 continue
-            self.all_available.append(name)
+            if name in self.indexed:
+                self.available.append(name)
+                self.pango_families.append(family)
         return
 
     def _get_indexed(self):
@@ -371,7 +390,7 @@ class Sort(object):
         Get a list of font families which belong to the "System".
         """
         for row in set(self.table.get('family', 'owner="System"')):
-            if row[0] in self.all_available:
+            if row[0] in self.available:
                 self.system.append(row[0])
         return
 
@@ -385,11 +404,17 @@ class Sort(object):
         self.manager.create_category(_('System'), families = self.system,
                                     comment = _('Fonts available to all users'))
         self.manager.create_category(_('User'), families = self.user,
-                comment = _('Fonts available only to %s' % USER.capitalize()))
+            comment = _('Fonts available only to {0}'.format(USER.capitalize())))
         self.manager.create_category(_('Orphans'), families = self.orphans,
                             comment = _('Fonts not present in any collection'))
         return
 
+    def _load_new_cache(self):
+        self.cache.close()
+        delete_cache()
+        self.cache = shelve.open(CACHE_FILE, protocol=cPickle.HIGHEST_PROTOCOL)
+        return
+
     def _load_user_collections(self):
         """
         Load any saved user collections
@@ -401,47 +426,36 @@ class Sort(object):
         """
         Load details for all available font families as reported by Pango.
         """
-        context = self.widget.get_pango_context()
-        pango_families = context.list_families()
-        self.total = len(pango_families)
-        for family in pango_families:
+        for family in self.pango_families:
             name = family.get_name()
-            if name in ALIAS_FAMILIES:
-                continue
             if self.cache.has_key(name):
                 obj = self.cache[name]
             else:
                 obj = Family(name)
+                obj.pango_family = PangoFamily(family)
                 if name in self.user:
                     obj.user = True
-                obj.styles = self._build_styles_dict(name)
-                obj.pango_family = PangoFamily(family)
+                rows = self.table.get('*', 'family="{0}"'.format(name))
+                for face in obj.pango_family.list_faces():
+                    for row in rows:
+                        if row['pdescr'] == face.description:
+                            obj.styles[face.get_face_name()] = FileDetails(row)
+                            break
                 self.cache[name] = obj
             self.manager[name] = obj
             self.processed += 1
             if self.progress_callback:
-                self.progress_callback(name, self.total, self.processed)
+                self.progress_callback(_("Loading available fonts..."),
+                                        self.total, self.processed)
         return
 
-    def total_families(self):
-        """
-        Total number of font families.
-        """
-        return len(self.available)
-
-    def total_files(self):
-        """
-        Total number of font files in database.
-        """
-        return self.file_total
-
 
 def _on(name):
     """
     Return a label suitable for display in a gtk.TreeView.
     """
     label = glib.markup_escape_text(name)
-    return '<span weight="heavy">%s</span>' % label
+    return '<span weight="heavy">{0}</span>'.format(label)
 
 def _off(name, strike = True):
     """
@@ -449,9 +463,10 @@ def _off(name, strike = True):
     """
     label = glib.markup_escape_text(name)
     if strike:
-        return '<span weight="ultralight" strikethrough="true">%s</span>' % label
+        return \
+        '<span weight="ultralight" strikethrough="true">{0}</span>'.format(label)
     else:
-        return '<span weight="ultralight">%s</span>' % label
+        return '<span weight="ultralight">{0}</span>'.format(label)
 
 
 def _set_library_permissions(library = USER_LIBRARY_DIR):
@@ -491,8 +506,7 @@ def do_font_install(filepath, library = USER_LIBRARY_DIR, cleanup = False):
     """
     name = basename(filepath)
     newpath = join(library, name[0].upper())
-    if not exists(newpath):
-        os.mkdir(newpath)
+    exists(newpath) or os.mkdir(newpath)
     shutil.copy(filepath, newpath)
     if filepath.endswith(T1_EXTS):
         metrics = splitext(filepath)[0] + '.*'
@@ -516,8 +530,7 @@ def install_font_archive(filepath, library = USER_LIBRARY_DIR, cleanup = False):
     #     self.notify()
     # ;-p
     newpath = join(library, dir_name[0].upper())
-    if not exists(newpath):
-        os.mkdir(newpath)
+    exists(newpath) or os.mkdir(newpath)
     shutil.move(arch_dir, newpath)
     shutil.rmtree(tmp_dir)
     if cleanup:
@@ -544,4 +557,3 @@ def do_library_cleanup(library = USER_LIBRARY_DIR):
     _set_library_permissions(library)
     _mkfontdirs(library)
     return
-
