@@ -31,11 +31,12 @@ import os
 import gtk
 import gobject
 
-from os.path import exists, join, isdir
+from os.path import join, isdir
 
 from core import database
-from utils.common import match, natural_sort, search
-from constants import HOME, DATABASE_FILE, CACHE_FILE
+from utils.common import delete_cache, delete_database, match, \
+                            natural_sort, search, run_dialog
+from constants import HOME
 
 
 EXTS = ('.ttf', '.ttc', '.otf', '.pfb', '.pfa', '.pfm', '.afm', '.bdf',
@@ -55,9 +56,9 @@ class PreferencesDialog():
         'AutoScanButton', 'PrefsInfoButton', 'PrefsAutoStart', 'PrefsHidden',
         'PrefsMinimize', 'PrefsOrphans', 'PrefsPangram', 'PrefsFontSize',
         'DBTree', 'UserDirTree', 'SearchDB', 'RemoveFromDB', 'ResetDB',
-        'PrefsClose', 'DefaultFolderBox', 'ArchTypeBox', 'PrefsFileChooser',
+        'DefaultFolderBox', 'ArchTypeBox', 'PrefsFileChooser',
         'PrefsInvalidDir', 'AutoScanInfo', 'PrefsCollFocus', 'PrefsCollTotal',
-        'PrefsFamTotal', 'PrefsToolTips', 'PrefsBrowseMode', 'PrefsLocSample',
+        'PrefsFamTotal', 'PrefsToolTips', 'PrefsLocSample',
         'PrefsJanitor'
         )
     def __init__(self, objects):
@@ -102,8 +103,7 @@ class PreferencesDialog():
                     'AutoScanButton'    :   self._on_autoscan,
                     'PrefsInfoButton'   :   self._on_info,
                     'RemoveFromDB'      :   self._on_del_db_item,
-                    'ResetDB'           :   self._on_reset_db,
-                    'PrefsClose'        :   self._on_close
+                    'ResetDB'           :   self._on_reset_db
                     }
         _toggled = {
                     'PrefsAutoStart'    :   self._on_autostart,
@@ -115,7 +115,6 @@ class PreferencesDialog():
                     'PrefsCollTotal'    :   self._on_coll_totals,
                     'PrefsFamTotal'     :   self._on_fam_totals,
                     'PrefsToolTips'     :   self._on_show_tooltips,
-                    'PrefsBrowseMode'   :   self._on_browse_mode,
                     'PrefsLocSample'    :   self._on_use_localized_sample,
                     'PrefsJanitor'      :   self._on_enable_janitor
                     }
@@ -154,21 +153,13 @@ class PreferencesDialog():
         Display file chooser dialog so user can add new directories
         to fontconfig search path.
         """
-        dialog = self.widgets['PrefsFileChooser']
-        dialog.set_current_folder(HOME)
-        response = dialog.run()
-        if response:
-            directory = dialog.get_filename()
-            dialog.hide()
-            while gtk.events_pending():
-                gtk.main_iteration()
+        self.widgets['PrefsFileChooser'].set_current_folder(HOME)
+        if run_dialog(dialog = self.widgets['PrefsFileChooser']):
+            directory = self.widgets['PrefsFileChooser'].get_filename()
             if directory == '/' or directory.startswith(BAD_PATHS):
                 self._on_bad_path()
             else:
                 return directory
-        dialog.hide()
-        while gtk.events_pending():
-            gtk.main_iteration()
         return
 
     def _get_selected_dir(self):
@@ -225,22 +216,10 @@ class PreferencesDialog():
         """
         Display warning dialog.
         """
-        dialog = self.widgets['PrefsInvalidDir']
-        dialog.run()
-        dialog.hide()
+        run_dialog(dialog = self.widgets['PrefsInvalidDir'])
         self._on_add_dir()
         return
 
-    def _on_browse_mode(self, widget):
-        switch = widget.get_active()
-        self.preferences.browsemode = switch
-        if switch:
-            self.update_required = True
-        else:
-            if self.objects['BrowseFonts'].get_property('visible'):
-                self.objects['BrowseFonts'].hide()
-        return
-
     def _on_clear_db_search(self, widget, unused_icon_pos, unused_event):
         """
         Clear database search entry.
@@ -331,7 +310,7 @@ class PreferencesDialog():
         except TypeError:
             return
         fonts = database.Table('Fonts')
-        query = 'family="%s"' % family
+        query = 'family="{0}"'.format(family)
         fonts.remove(query)
         fonts.close()
         db_model.remove(treeiter)
@@ -407,9 +386,7 @@ class PreferencesDialog():
         """
         Display info dialog.
         """
-        dialog = self.widgets['AutoScanInfo']
-        dialog.run()
-        dialog.hide()
+        run_dialog(dialog = self.widgets['AutoScanInfo'])
         return
 
     def _on_hide_at_start(self, widget):
@@ -434,10 +411,8 @@ class PreferencesDialog():
         db_model = db_tree.get_model()
         db_model.clear()
         db_tree.queue_draw()
-        if exists(CACHE_FILE):
-            os.unlink(CACHE_FILE)
-        if exists(DATABASE_FILE):
-            os.unlink(DATABASE_FILE)
+        delete_cache()
+        delete_database()
         self.update_required = True
         return
 
@@ -565,7 +540,6 @@ class PreferencesDialog():
                     'PrefsCollTotal'    :   'collectiontotals',
                     'PrefsFamTotal'     :   'familytotals',
                     'PrefsToolTips'     :   'tooltips',
-                    'PrefsBrowseMode'   :   'browsemode',
                     'PrefsLocSample'    :   'localized',
                     'PrefsJanitor'      :   'janitor'
                     }
@@ -592,4 +566,3 @@ class PreferencesDialog():
                 }
         self.widgets['ArchCombo'].set_active(types[arch_type])
         return
-
