@@ -32,25 +32,32 @@ import glib
 import gobject
 import logging
 import subprocess
-import time
 
 from os.path import basename
 
 from constants import COMPARE_TEXT, DEFAULT_STYLES, PREVIEW_TEXT, \
                         STANDARD_TEXT, LOCALIZED_TEXT
-from fontinfo import FontInformation
-from utils.common import correct_slider_behavior
+from ui.fontinfo import FontInformation
+from utils.common import begin_drag, correct_slider_behavior
+
+TARGET_TYPE_BROWSE_ROW = 30
+TARGET_TYPE_COMPARE_ROW = 50
+
+BROWSE_DRAG_TARGETS = [('browser_add', gtk.TARGET_SAME_APP, TARGET_TYPE_BROWSE_ROW)]
+BROWSE_DRAG_ACTIONS = (gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
+COMPARE_DRAG_TARGETS = [('reorder', gtk.TARGET_SAME_WIDGET, TARGET_TYPE_COMPARE_ROW)]
+COMPARE_DRAG_ACTIONS = (gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
 
 
 def set_preview_text(use_localized_sample):
     global PREVIEW_TEXT
     global COMPARE_TEXT
     if use_localized_sample:
-        PREVIEW_TEXT = PREVIEW_TEXT % LOCALIZED_TEXT
-        COMPARE_TEXT = COMPARE_TEXT % LOCALIZED_TEXT
+        PREVIEW_TEXT = PREVIEW_TEXT.format(LOCALIZED_TEXT)
+        COMPARE_TEXT = COMPARE_TEXT.format(LOCALIZED_TEXT)
     else:
-        PREVIEW_TEXT = PREVIEW_TEXT % STANDARD_TEXT
-        COMPARE_TEXT = COMPARE_TEXT % STANDARD_TEXT
+        PREVIEW_TEXT = PREVIEW_TEXT.format(STANDARD_TEXT)
+        COMPARE_TEXT = COMPARE_TEXT.format(STANDARD_TEXT)
     return
 
 
@@ -86,12 +93,12 @@ class Previews(object):
         # Make it do something
         size_adjustment.connect('value-changed', self._on_size_adj_v_change)
         # Correct slider behavior - up means up, down means down
-        self.objects['FontSizeSlider'].connect('scroll-event', 
+        self.objects['FontSizeSlider'].connect('scroll-event',
                                                 correct_slider_behavior, 1.0)
         # Gnome Character Map
         character_map = self.objects['CharacterMap']
         character_map.connect('clicked', self.on_char_map)
-        if not 'gucharmap' in self.objects['AvailableApps']:
+        if not 'gucharmap' in self.objects['AppCache']:
             character_map.set_sensitive(False)
             character_map.set_tooltip_text(_('This feature requires Gucharmap'))
         self.objects['StyleCombo'].connect('changed', self._on_style_changed)
@@ -110,6 +117,14 @@ class Previews(object):
         self.compare_tree.append_column(preview_column)
         compare_selection = self.compare_tree.get_selection()
         compare_selection.set_select_function(self._set_preview_row_selection)
+        self.compare_tree.enable_model_drag_dest(COMPARE_DRAG_TARGETS,
+                                                    COMPARE_DRAG_ACTIONS)
+        self.compare_tree.enable_model_drag_source\
+                                (gtk.gdk.BUTTON1_MASK | gtk.gdk.RELEASE_MASK,
+                                    COMPARE_DRAG_TARGETS, COMPARE_DRAG_ACTIONS)
+        self.compare_tree.connect('drag-data-received',
+                                            self._on_drag_data_received)
+        self.compare_tree.connect_after('drag-begin', self._on_begin_drag)
         self.objects['AddToCompare'].connect('clicked', self._on_add_compare)
         self.objects['RemoveFromCompare'].connect('clicked',
                                                     self._on_remove_compare)
@@ -122,18 +137,87 @@ class Previews(object):
     def _on_browse_fonts(self, widget):
         if widget == self.objects['BrowseFonts']:
             self.objects['MainNotebook'].set_current_page(1)
-            self.objects['CollectionButtonsFrame'].hide()
             collection = self.objects['Treeviews'].current_collection
             families = self.manager.list_families_in(collection)
-            self.objects['Browse'].update_tree(families)
+            self.objects['FontBrowser'].update_tree(families)
+            self.objects['Treeviews'].set_direct_select()
         else:
-            if self.objects['Browse'].working:
-                self.objects['Browse'].cancel = True
+            # Clearing the font browser prevents the interface from hanging
+            self.objects['FontBrowser'].update_tree([])
             self.objects['MainNotebook'].set_current_page(0)
-            self.objects['CollectionButtonsFrame'].show()
             self.objects['Treeviews'].update_views(True)
-        while gtk.events_pending():
-            gtk.main_iteration()
+            self.objects['Treeviews'].set_indirect_select()
+        return
+
+    def _on_begin_drag(self, widget, context):
+        model, treeiter = self.compare_tree.get_selection().get_selected()
+        font = model.get_value(treeiter, 0)
+        win = gtk.Window(gtk.WINDOW_POPUP)
+        preview = gtk.Label()
+        preview.set_markup('<span size="x-large"> {0} \n\n</span><span \
+                    font_desc="{1}" size="xx-large"> {2} </span>'.format(
+                    font, font, self.compare_text))
+        preview.set_padding(4, 4)
+        win.add(preview)
+        win.set_title('You should NOT be seeing this window frame!')
+        win.set_decorated(False)
+        win.set_opacity(0.95)
+        win.show_all()
+        win.set_transient_for(self.objects['MainWindow'])
+        context.set_icon_widget(win, 0, 0)
+        return
+
+    # Gets really ugly here dragging and dropping two separate rows...
+    # Seems to work though :-D
+    def _on_drag_data_received(self, widget, context, x, y, data, info, tstamp):
+        assert(info == TARGET_TYPE_COMPARE_ROW)
+        treeiter = widget.get_selection().get_selected()[1]
+        model = widget.get_model()
+        orig_path = model.get_path(treeiter)[0]
+        assert(orig_path % 2 == 0)
+        data = model.get(treeiter, 0, 1)
+        drop_info = widget.get_dest_row_at_pos(x, y)
+        if drop_info:
+            path, position = drop_info
+            if path[0] == orig_path or path[0] - 2 == orig_path \
+            or path[0] - 1 == orig_path:
+                return
+            if position == gtk.TREE_VIEW_DROP_BEFORE:
+                if path[0] % 2 != 0:
+                    return
+                try:
+                    treeiter = model.get_iter(path)
+                    del model[orig_path + 1]
+                    del model[orig_path]
+                    treeiter = model.insert_before(treeiter, data)
+                    treeiter = model.insert_before(treeiter, data)
+                except ValueError:
+                    return
+            elif position == gtk.TREE_VIEW_DROP_AFTER:
+                if path[0] % 2 == 0:
+                    return
+                try:
+                    treeiter = model.get_iter(path)
+                    del model[orig_path + 1]
+                    del model[orig_path]
+                    treeiter = model.insert_after(treeiter, data)
+                    treeiter = model.insert_before(treeiter, data)
+                except ValueError:
+                    del model[orig_path + 1]
+                    del model[orig_path]
+                    treeiter = model.append(data)
+                    treeiter = model.append(data)
+            else:
+                return
+        else:
+            del model[orig_path + 1]
+            del model[orig_path]
+            treeiter = model.append(data)
+            treeiter = model.append(data)
+        path = model.get_path(treeiter)
+        if path:
+            widget.get_selection().select_path(path)
+        self.update_compare_view()
         return
 
     def _on_switch_mode(self, unused_widget):
@@ -216,7 +300,7 @@ class Previews(object):
 
     def _cell_data_cb(self, column, cell, model, treeiter):
         style = self.compare_tree.get_style()
-        # pango.SCALE is currently 9216
+        # pango.SCALE doesn't work as expected, this does...
         scale = 9216
         font_desc = style.font_desc
         font_desc.set_size(int(scale * 1.2))
@@ -298,9 +382,9 @@ class Previews(object):
                                 lambda widget, event: widget.hide() or True)
         self.objects['CloseColorSelector'].connect('clicked', \
                                         lambda widget: dialog.hide() or True)
-        self.objects['ForegroundColor'].set_property('color', 
+        self.objects['ForegroundColor'].set_property('color',
                                                         self.preview_fgcolor)
-        self.objects['BackgroundColor'].set_property('color', 
+        self.objects['BackgroundColor'].set_property('color',
                                                         self.preview_bgcolor)
         self.objects['ForegroundColor'].connect('color-set',
                                                         self._on_colors_changed)
@@ -355,9 +439,9 @@ class Previews(object):
         try:
             logging.info("Launching GNOME Character Map")
             font = self.current_style.to_string()
-            subprocess.Popen(['gucharmap', '--font=%s 22' % font])
+            subprocess.Popen(['gucharmap', '--font={0} 22'.format(font)])
         except OSError, error:
-            logging.error("Error: %s" % error)
+            logging.error("Error: {0}".format(error))
         return
 
     def _on_font_changed(self, treeselection):
@@ -377,7 +461,7 @@ class Previews(object):
         if self.current_family.enabled:
             enable_font.set_sensitive(False)
             disable_font.set_sensitive(True)
-            if 'gucharmap' in self.objects['AvailableApps']:
+            if 'gucharmap' in self.objects['AppCache']:
                 character_map.set_sensitive(True)
         else:
             enable_font.set_sensitive(True)
@@ -395,9 +479,10 @@ class Previews(object):
             return
         style = widget.get_active_text()
         self.current_style_as_string = style
-        valid_styles = 'Bold', 'Bold Italic', 'Italic'
+        valid_styles = ('Bold', 'Bold Italic', 'Italic',
+                        _('Bold'), _('Bold Italic'), _('Italic'))
         if self.current_family.enabled and \
-        'gucharmap' in self.objects['AvailableApps']:
+        'gucharmap' in self.objects['AppCache']:
             if style in self.current_family.styles.iterkeys() or \
             style in valid_styles:
                 character_map.set_sensitive(True)
@@ -424,7 +509,7 @@ class Previews(object):
         style_combo.get_model().clear()
         family = self.current_family.pango_family
         selected_face = None
-        active = -1
+        active = None
         i = 0
         for face in family.list_faces():
             style_combo.append_text(face.get_face_name())
@@ -433,6 +518,8 @@ class Previews(object):
                     selected_face = face
                     active = i
             i += 1
+        if active is None:
+            active = 0
         style_combo.set_active(active)
         if selected_face:
             descr = selected_face.describe()
@@ -463,20 +550,19 @@ class Previews(object):
         size = self.size
         tag = t_buffer.create_tag(None, font_desc=descr, size_points=size,
                                     wrap_mode=gtk.WRAP_WORD)
-        t_buffer.insert(t_buffer.get_end_iter(), descr.to_string() + '\n')
+        t_buffer.insert(t_buffer.get_end_iter(),
+                        '{0} {1}\n'.format(self.current_family.get_name(),
+                                            self.current_style_as_string))
         t_buffer.insert(t_buffer.get_end_iter(), '\n' + self.preview_text)
         t_buffer.apply_tag(tag, t_buffer.get_start_iter(),
                                     t_buffer.get_end_iter())
-
         style_combo = self.objects['StyleCombo']
         style = style_combo.get_model()[style_combo.get_active()][0]
         if not style in self.current_family.styles.iterkeys():
-            tooltip = \
-            _('Style provided by the rendering library.')
+            tooltip = _('Style provided by the rendering library.')
         else:
             filepath = self.current_family.styles[style]['filepath']
-            tooltip = \
-            _('Style provided by %s' % basename(filepath))
+            tooltip = _('Style provided by {0}'.format(basename(filepath)))
         style_combo.set_tooltip_text(tooltip)
         if self.info_dialog:
             if self.info_dialog.window.get_property('visible'):
@@ -489,12 +575,16 @@ class Previews(object):
         """
         if not self.info_dialog:
             self.info_dialog = FontInformation(self.objects)
-        self.info_dialog.show(self.current_family, self.current_style,
-                                        self.current_style_as_string)
+        try:
+            self.info_dialog.show(self.current_family.styles\
+                                [self.current_style_as_string]['filepath'],
+                                self.current_style)
+        except KeyError:
+            self.info_dialog.show(None, None)
         return
 
 
-class Browse(object):
+class FontBrowser(object):
     """
     Create a treeview containing inline previews of all fonts.
     """
@@ -503,35 +593,43 @@ class Browse(object):
         self.browse_tree = self.objects['BrowseTree']
         self.families = self.objects['FontManager'].list_families()
         self.treestore = gtk.TreeStore(gobject.TYPE_STRING,
-                                    gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN)
+                                    gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN,
+                                    gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
         self.browse_tree.set_model(self.treestore)
         renderer = gtk.CellRendererText()
         renderer.set_property('ypad', 3)
         column = gtk.TreeViewColumn(None, renderer)
         column.set_cell_data_func(renderer, self._cell_data_cb)
         self.browse_tree.append_column(column)
-        self.working = False
-        self.cancel = False
+        self.browse_tree.get_selection().set_select_function(self._on_select)
+        self.browse_tree.enable_model_drag_source\
+                                (gtk.gdk.BUTTON1_MASK | gtk.gdk.RELEASE_MASK,
+                                    BROWSE_DRAG_TARGETS, BROWSE_DRAG_ACTIONS)
+        self.browse_tree.connect_after('drag-begin', begin_drag)
         self.size = self.objects['Preferences'].previewsize
         size_adjustment = self.objects['SizeAdjustment']
         # Make it do something
         size_adjustment.connect('value-changed', self.update_browse_view)
         # Correct slider behavior - up means up, down means down
-        self.objects['BrowseSizeSlider'].connect('scroll-event', 
+        self.objects['BrowseSizeSlider'].connect('scroll-event',
                                                 correct_slider_behavior, 1.0)
 
-    def update_tree(self, families):
+    def _on_select(self, path):
+        model = self.browse_tree.get_model()
+        if model.get_value(model.get_iter(path), 3):
+            return True
+        else:
+            self.browse_tree.get_selection().select_path(path[0])
+            return False
+
+    def update_tree(self, families, scroll_to_top = True):
         self.families = families
         self.objects['BrowseScroll'].set_sensitive(False)
-        if self.working:
-            self.cancel = True
-        while self.working:
-            time.sleep(0.25)
-            while gtk.events_pending():
-                gtk.main_iteration()
-            continue
         self.treestore.clear()
-        gobject.idle_add(self._populate_browse_tree)
+        self._populate_browse_tree()
+        self.objects['BrowseScroll'].set_sensitive(True)
+        if scroll_to_top:
+            self.browse_tree.scroll_to_point(0, 0)
         return
 
     def _cell_data_cb(self, column, cell, model, treeiter):
@@ -555,40 +653,22 @@ class Browse(object):
             cellset('strikethrough', model.get_value(treeiter, 2))
         return
 
-    # Adapted from gnome-specimen.
     def _populate_browse_tree(self):
-        """
-        Idle callback that adds font families to the tree model
-        """
-        # loading is done when the list of remaining families is empty
-        if len(self.families) == 0 or self.cancel:
-            self.objects['BrowseScroll'].set_sensitive(True)
-            self.browse_tree.expand_all()
-            self.working = False
-            self.cancel = False
-            return False
-        self.working = True
-        # speedup: temporarily disconnect the model
-        model = self.browse_tree.get_model()
+        # temporarily disconnect the model
         self.browse_tree.set_model(None)
-        # add a bunch of fonts and faces to the treemodel
-        try:
-            for unused_i in range(100):
-                family = self.families.pop(0)
-                obj = self.objects['FontManager'][family]
-                root_node = self.treestore.append(None,
-                            [glib.markup_escape_text(family), None, False])
-                for style in obj.pango_family.list_faces():
-                    displaytext = '%s %s' % (glib.markup_escape_text(family),
-                    glib.markup_escape_text(style.get_face_name()))
-                    self.treestore.append(root_node, [displaytext,
-                    style.describe(), not obj.enabled])
-        except IndexError:
-            pass
+        for family in self.families:
+            obj = self.objects['FontManager'][family]
+            root_node = self.treestore.append(None,
+                [glib.markup_escape_text(family), None, False, True, family])
+            for style in obj.pango_family.list_faces():
+                displaytext = '{0} {1}'.format(glib.markup_escape_text(family),
+                            glib.markup_escape_text(style.get_face_name()))
+                self.treestore.append(root_node,
+                [displaytext, style.describe(), not obj.enabled, False, family])
         # reconnect the model
-        self.browse_tree.set_model(model)
-        # run again
-        return True
+        self.browse_tree.set_model(self.treestore)
+        self.browse_tree.expand_all()
+        return
 
     def update_browse_view(self, widget):
         """
