@@ -36,22 +36,24 @@ import urlparse
 
 from os.path import join
 
-import _fontutils
+import fontutils
 
-from actions import UserActions
+from ui.actions import UserActions
 from core.database import Table
 from constants import PACKAGE_DATA_DIR, FONT_EXTS
-from custom import CellRendererTotal
-from library import InstallFonts
-from utils.common import match, natural_sort, search
+from ui.custom import CellRendererTotal
+from ui.library import InstallFonts
+from utils.common import begin_drag, match, natural_sort, search, run_dialog
 
 TARGET_TYPE_COLLECTION_ROW = 10
 TARGET_TYPE_FAMILY_ROW = 20
-TARGET_TYPE_EXTERNAL_DROP = 30
+TARGET_TYPE_BROWSE_ROW = 30
+TARGET_TYPE_EXTERNAL_DROP = 40
 
 COLLECTION_DRAG_TARGETS = [
                 ('reorder', gtk.TARGET_SAME_WIDGET, TARGET_TYPE_COLLECTION_ROW),
-                ('add', gtk.TARGET_SAME_APP, TARGET_TYPE_FAMILY_ROW)
+                ('add', gtk.TARGET_SAME_APP, TARGET_TYPE_FAMILY_ROW),
+                ('browser_add', gtk.TARGET_SAME_APP, TARGET_TYPE_BROWSE_ROW)
                 ]
 COLLECTION_DRAG_ACTIONS = (gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
 
@@ -65,13 +67,13 @@ class Treeviews(object):
     _types = {
                 'TrueType'      :   'truetype.png',
                 'Type 1'        :   'type1.png',
-                'BDF'           :   'bitmap.png',
-                'PCF'           :   'bitmap.png',
+                'BDF'           :   'bdf.png',
+                'PCF'           :   'pcf.png',
                 'Type 42'       :   'type42.png',
                 'CID Type 1'    :   'type1.png',
                 'CFF'           :   'opentype.png',
-                'PFR'           :   'bitmap.png',
-                'Windows FNT'   :   'bitmap.png'
+                'PFR'           :   'pfr.png',
+                'Windows FNT'   :   'fnt.png'
                 }
     def __init__(self, objects):
         self.objects = objects
@@ -143,6 +145,8 @@ class Treeviews(object):
         category_treeselect = self.category_tree.get_selection()
         category_treeselect.set_select_function(lambda path: len(path) == 2)
         category_treeselect.connect('changed', self._on_collection_selected)
+        category_treeselect.connect('changed', self.__on_collection_selected)
+        category_treeselect.handler_block_by_func(self.__on_collection_selected)
         if self.objects['Preferences'].tooltips:
             self.category_tree.set_tooltip_column(2)
         self._load_categories()
@@ -171,6 +175,8 @@ class Treeviews(object):
         collection_treeselect = self.collection_tree.get_selection()
         collection_treeselect.set_select_function(lambda path: len(path) == 2)
         collection_treeselect.connect('changed', self._on_collection_selected)
+        collection_treeselect.connect('changed', self.__on_collection_selected)
+        collection_treeselect.handler_block_by_func(self.__on_collection_selected)
         if self.objects['Preferences'].tooltips:
             self.collection_tree.set_tooltip_column(2)
         self.collection_tree.enable_model_drag_dest(COLLECTION_DRAG_TARGETS,
@@ -186,8 +192,8 @@ class Treeviews(object):
 
     def _setup_families(self):
         family_header = gtk.Label()
-        family_header.set_markup('<span size="large" weight="heavy">%s</span>'\
-                                                                % _('Family'))
+        family_header.set_markup(
+            '<span size="large" weight="heavy">{0}</span>'.format(_('Family')))
         family_model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                                                             gobject.TYPE_STRING)
         if self.objects['Preferences'].experimental:
@@ -208,7 +214,7 @@ class Treeviews(object):
         family_treeselect.set_mode(gtk.SELECTION_MULTIPLE)
         self.family_tree.connect('button-press-event', self._on_button_press)
         self.family_tree.connect('button-release-event', self._on_button_release)
-        self.family_tree.connect_after('drag-begin', _begin_drag)
+        self.family_tree.connect_after('drag-begin', begin_drag)
         self.family_tree.connect('row-activated', self._on_family_activated)
         self.family_tree.connect('query-tooltip', self._on_family_tooltip)
         self.family_tree.append_column(family_column)
@@ -228,10 +234,9 @@ class Treeviews(object):
         Display "Advanced Search" dialog.
         """
         filt = self.filter.run()
-        if filt:
+        if isinstance(filt, list):
             self._show_collection(filt)
-        else:
-            return
+        return
 
     # Disable warnings related to invalid names
     # pylint: disable-msg=C0103
@@ -256,17 +261,16 @@ class Treeviews(object):
                         in data.data.splitlines() if path.endswith(FONT_EXTS)]
         block = _('All'), _('System'), _('User'), _('Orphans')
         if self.current_collection not in block:
-            families = [ _fontutils.FT_Get_File_Info(path)['family'] \
+            families = [ fontutils.FT_Get_File_Info(path)['family'] \
                         for path in filelist]
-            self.manager.add_families_to(self.current_collection, families)
+            if families:
+                self.manager.add_families_to(self.current_collection, families)
         if not self.objects['Main'].installer:
             self.objects['Main'].installer = InstallFonts(self.objects)
         self.objects['Main'].installer.process_install(filelist)
         return
 
     def _on_drag_data_received(self, widget, context, x, y, data, info, tstamp):
-        if self.objects['MainNotebook'].get_current_page() != 0:
-            return
         model = widget.get_model()
         root = model.get_iter_root()
         if not root:
@@ -298,6 +302,18 @@ class Treeviews(object):
             self.manager.add_families_to(collection, families)
             if self.objects['Preferences'].focusondrop:
                 widget.get_selection().select_path(path)
+        elif info == TARGET_TYPE_BROWSE_ROW:
+            titer = context.get_source_widget().get_selection().get_selected()[1]
+            family = context.get_source_widget().get_model().get_value(titer, 4)
+            collection = model.get(model.get_iter(
+                                    widget.get_path_at_pos(x,y)[0]), 0, 1)[0]
+            self.manager.add_families_to(collection, family)
+            self.manager.auto_enable_collections()
+            self._update_collection_treeview()
+            self.update_category_treeview()
+            families = self.manager.list_families_in(self.current_collection)
+            self.objects['FontBrowser'].update_tree(families, False)
+            return
         self.update_views()
         return
 
@@ -318,12 +334,12 @@ class Treeviews(object):
             family = self.manager[name].pango_family
             famname = glib.markup_escape_text(name)
             markup = \
-            '\n\t<span weight="heavy" size="large">%s</span>\t\t\n\n' % famname
+    '\n\t<span weight="heavy" size="large">{0}</span>\t\t\n\n'.format(famname)
             for face in family.list_faces():
                 facename = glib.markup_escape_text(face.get_face_name())
                 facedescr = glib.markup_escape_text(face.describe().to_string())
                 subs = (facedescr, facename)
-                markup += '<span font_desc="%s">\t%s\t\t</span>\n' % subs
+                markup += '<span font_desc="{0}">\t{1}\t\t</span>\n'.format(*subs)
             tooltip.set_markup(markup)
             icon = self._get_type_icon(name)
             tooltip.set_icon(icon)
@@ -387,8 +403,6 @@ class Treeviews(object):
         """
         Display context menu when a family is right clicked.
         """
-        menu = gtk.Menu()
-        actions = self.actions.actions
         selection = widget.get_selection()
         if selection.count_selected_rows() > 1:
             return
@@ -403,6 +417,21 @@ class Treeviews(object):
             filepath = self.manager[family].styles[style]['filepath']
         except KeyError:
             return
+        menu = gtk.Menu()
+        secondary = gtk.Menu()
+        collections = self.manager.list_collections()
+        if collections:
+            menuitem = gtk.MenuItem(_('Add to'))
+            menu.append(menuitem)
+            menuitem.set_submenu(secondary)
+            top_separator = gtk.SeparatorMenuItem()
+            menu.append(top_separator)
+            for collection in natural_sort(collections):
+                menuitem = gtk.MenuItem(collection)
+                secondary.append(menuitem)
+                menuitem.connect('activate', self._add_on_right_click,
+                                                    collection, family)
+        actions = self.actions.actions
         for action in self.actions.get_actions():
             menuitem = gtk.MenuItem(actions[action]['name'])
             menuitem.set_name(actions[action]['name'])
@@ -411,8 +440,8 @@ class Treeviews(object):
             menu.append(menuitem)
             menuitem.connect('activate', self.actions.run_command,
                                         (filepath, family, style))
-        separator = gtk.SeparatorMenuItem()
-        menu.append(separator)
+        bottom_separator = gtk.SeparatorMenuItem()
+        menu.append(bottom_separator)
         menuitem = gtk.MenuItem(_('Edit actions...'))
         menuitem.connect('activate', self.actions.run)
         menu.append(menuitem)
@@ -422,6 +451,12 @@ class Treeviews(object):
             gtk.main_iteration()
         return
 
+    def _add_on_right_click(self, unused_menuitem, collection, family):
+        self.manager.add_families_to(collection, family)
+        self.manager.auto_enable_collections()
+        self._update_collection_treeview()
+        return
+
     def _load_categories(self):
         category_model = self.category_tree.get_model()
         header = category_model.append(None,
@@ -464,9 +499,32 @@ class Treeviews(object):
             del collections[old_name]
             model.set(treeiter, 0, new_name)
             model.set(treeiter, 1, collections[new_name].get_label())
+        self.current_collection = new_name
         return
 
+    # http://code.google.com/p/font-manager/issues/detail?id=41
     def _on_collection_selected(self, tree_selection):
+        glib.idle_add(self.__on_collection_selected, tree_selection)
+        return
+
+    # Sometimes we need our selection to block...
+    def set_direct_select(self):
+        direct = self.__on_collection_selected
+        indirect = self._on_collection_selected
+        for widget in self.category_tree, self.collection_tree:
+            widget.get_selection().handler_block_by_func(indirect)
+            widget.get_selection().handler_unblock_by_func(direct)
+        return
+
+    def set_indirect_select(self):
+        direct = self.__on_collection_selected
+        indirect = self._on_collection_selected
+        for widget in self.category_tree, self.collection_tree:
+            widget.get_selection().handler_block_by_func(direct)
+            widget.get_selection().handler_unblock_by_func(indirect)
+        return
+
+    def __on_collection_selected(self, tree_selection):
         try:
             model, treeiter = tree_selection.get_selected()
             self.current_collection = model.get_value(treeiter, 0)
@@ -600,11 +658,7 @@ class Treeviews(object):
         return
 
     def _modify_mad_fonts(self):
-        dialog = self.objects['MadFontsWarning']
-        result = dialog.run()
-        dialog.hide()
-        while gtk.events_pending():
-            gtk.main_iteration()
+        result = run_dialog(dialog = self.objects['MadFontsWarning'])
         return (result == gtk.RESPONSE_OK)
 
     def _update_sensitivity(self, treeview):
@@ -624,11 +678,9 @@ class Treeviews(object):
         families = self.manager.list_families_in(self.current_collection)
         collection = self.current_collection
         for widget in widgets:
-            if collection in block:
+            if collection in block or collection == _('User'):
                 widget.set_sensitive(False)
-            elif collection == _('User'):
-                widget.set_sensitive(False)
-                if widget == widgets[1]:
+                if collection == _('User') and widget == widgets[1]:
                     widget.set_sensitive(True)
             else:
                 widget.set_sensitive(True)
@@ -639,11 +691,14 @@ class Treeviews(object):
         return
 
     def _on_new_collection(self, unused_widget):
+        current_page = self.objects['MainNotebook'].get_current_page()
+        if current_page == 0:
+            self.set_direct_select()
         new_name = _('New Collection')
         alt = 0
         while new_name in self.manager.list_collections():
             alt += 1
-            new_name =  _('New Collection %s' % alt)
+            new_name =  _('New Collection {0}').format(alt)
         collection = self.manager.create_collection(new_name)
         obj = self.manager.collections[new_name]
         model = self.collection_tree.get_model()
@@ -655,6 +710,8 @@ class Treeviews(object):
         column = self.collection_tree.get_column(0)
         self.collection_tree.scroll_to_cell(path, column)
         self.collection_tree.set_cursor(path, column, start_editing=True)
+        if current_page == 0:
+            self.set_indirect_select()
         return
 
     def _selected_paths(self):
@@ -697,12 +754,17 @@ class Treeviews(object):
         for family in families:
             try:
                 obj = self.manager[family]
-                family_model.append([obj.get_name(), obj.get_label(), 
+                family_model.append([obj.get_name(), obj.get_label(),
                                                             obj.get_count()])
             except KeyError:
-                logging.error('Could not find %s for user collection %s' %
-                                (family, self.current_collection))
-                logging.info('Skipping...')
+                logging.error(
+                'Could not find {0} for user collection {1}'.format(family,
+                                                    self.current_collection))
+                logging.warn(
+                'Removing {0} from user collection {1}'.format(family,
+                                                    self.current_collection))
+                self.manager.remove_families_from(self.current_collection,
+                                                    family)
                 continue
         self.family_tree.set_model(family_model)
         self.family_tree.thaw_child_notify()
@@ -755,8 +817,11 @@ class Treeviews(object):
         model = self.collection_tree.get_model()
         header = model.get_iter_first()
         treeiter = model.iter_children(header)
+        collections = self.manager.list_collections()
         while treeiter:
             name, label = model.get(treeiter, 0, 1)
+            while name in collections:
+                collections.remove(name)
             try:
                 obj = self.manager.collections[name]
             except KeyError:
@@ -768,6 +833,17 @@ class Treeviews(object):
                 model.set(treeiter, 1, new_label)
             model.set(treeiter, 3, str(len(obj.families)))
             treeiter = model.iter_next(treeiter)
+        if collections:
+            for collection in collections:
+                try:
+                    obj = self.manager.collections[collection]
+                except KeyError:
+                    continue
+            model = self.collection_tree.get_model()
+            header = model.get_iter_first()
+            treeiter = model.append(header, [obj.get_name(), obj.get_label(),
+                                        obj.comment, str(len(obj.families))])
+            self.collection_tree.expand_all()
         return
 
     def _update_family_treeview(self):
@@ -802,8 +878,11 @@ class Treeviews(object):
             self._update_family_treeview()
             self._show_collection()
         else:
-            families = self.manager.list_families_in(self.current_collection)
-            self.objects['Browse'].update_tree(families)
+            try:
+                families = self.manager.list_families_in(self.current_collection)
+                self.objects['FontBrowser'].update_tree(families)
+            except IndexError:
+                pass
         self.objects['MainWindow'].queue_draw()
         return
 
@@ -811,9 +890,22 @@ class Treeviews(object):
 class TreeviewFilter(object):
     _widgets = (
                 'SearchDialog', 'FamilyCombo', 'FamilyEntry', 'TypeCombo',
-                'TypeComboEntry', 'FoundryCombo', 'FoundryComboEntry',
-                'FilepathCombo', 'FilepathEntry'
+                'FoundryCombo', 'FoundryComboEntry', 'FilepathCombo',
+                'FilepathEntry', 'FiletypeCombo', 'FiletypeComboEntry'
                 )
+    _family_types = (
+                    'All', 'No Fit', 'Text and Display', 'Script', 'Decorative',
+                    'Pictorial'
+                    )
+    _query_opts = {
+                    0   :   '{0} LIKE "%{1}%"',
+                    1   :   '{0} LIKE "{1}%"',
+                    2   :   '{0} LIKE "%{1}"',
+                    3   :   '{0}="{1}"',
+                    4   :   '{0}!="{1}"'
+                    }
+    _combos = ('FamilyCombo', 'TypeCombo', 'FoundryCombo', 'FilepathCombo',
+                'FiletypeCombo')
     def __init__(self, objects):
         self.objects = objects
         self.builder = objects.builder
@@ -822,9 +914,6 @@ class TreeviewFilter(object):
             self.widgets[widget] = self.builder.get_object(widget)
         self.widgets['SearchDialog'].connect('delete-event', \
                                 lambda widget, event: widget.response(0))
-        self.combos = (
-                    'FamilyCombo', 'TypeCombo', 'FoundryCombo', 'FilepathCombo'
-                    )
         self._types = self._get_types()
         self._foundries = self._get_foundries()
         self._setup_combos()
@@ -834,77 +923,58 @@ class TreeviewFilter(object):
         """
         Return query results.
         """
-        family = self._get_family()
-        typ = self._get_type()
-        foundry = self._get_foundry()
-        filepath = self._get_filepath()
         query = ''
-        for entry in family, typ, foundry, filepath:
+        filters = (self._get_family(), self._get_type(), self._get_foundry(),
+                    self._get_filetype(), self._get_filepath())
+        for entry in filters:
             if entry:
                 if query == '':
-                    query = '%s' % entry
+                    query = '{0}'.format(entry)
                 else:
-                    query = '%s AND %s' % (query, entry)
+                    query = '{0} AND {1}'.format(query, entry)
         fonts = Table('Fonts')
-        filt = []
-        for row in set(fonts.get('family', query)):
-            filt.append(row[0])
+        filt = [row[0] for row in set(fonts.get('family', query))]
         fonts.close()
-        return filt
+        return natural_sort(filt)
 
     def _get_family(self):
+        active = self.widgets['FamilyCombo'].get_active()
         family = self.widgets['FamilyEntry'].get_text()
         if family != '':
-            active = self.widgets['FamilyCombo'].get_active()
-            like = 'family LIKE "'
-            if active == 0:
-                query = like + '%' + family + '%"'
-            elif active == 1:
-                query = like + family + '%"'
-            elif active == 2:
-                query = like + '%' + family + '"'
+            return self._query_opts[active].format('family', family)
         else:
-            query = False
-        return query
+            return False
 
     def _get_type(self):
-        typ = self.widgets['TypeComboEntry'].child.get_text()
-        if typ != '':
-            active = self.widgets['TypeCombo'].get_active()
-            if active == 0:
-                query = 'filetype="%s"' % typ
-            elif active == 1:
-                query = 'filetype!="%s"' % typ
+        active = self.widgets['TypeCombo'].get_active()
+        if active == 0:
+            return False
         else:
-            query = False
-        return query
+            return 'panose LIKE "{0}:%"'.format(str(active))
+
+    def _get_filetype(self):
+        active = self.widgets['FiletypeCombo'].get_active()
+        frmat = self.widgets['FiletypeComboEntry'].child.get_text()
+        if frmat != '':
+            return self._query_opts[active + 3].format('filetype', frmat)
+        else:
+            return False
 
     def _get_foundry(self):
+        active = self.widgets['FiletypeCombo'].get_active()
         foundry = self.widgets['FoundryComboEntry'].child.get_text()
         if foundry != '':
-            active = self.widgets['FoundryCombo'].get_active()
-            if active == 0:
-                query = 'foundry="%s"' % foundry
-            elif active == 1:
-                query = 'foundry!="%s"' % foundry
+            return self._query_opts[active + 3].format('foundry', foundry)
         else:
-            query = False
-        return query
+            return False
 
     def _get_filepath(self):
+        active = self.widgets['FilepathCombo'].get_active()
         filepath = self.widgets['FilepathEntry'].get_text()
         if filepath != '':
-            active = self.widgets['FilepathCombo'].get_active()
-            like = 'filepath LIKE "'
-            if active == 0:
-                query = like + '%' + filepath + '%"'
-            elif active == 1:
-                query = like + filepath + '%"'
-            elif active == 2:
-                query = like + '%' + filepath + '"'
+            return self._query_opts[active].format('filepath', filepath)
         else:
-            query = False
-        return query
+            return False
 
     def _get_foundries(self):
         """
@@ -924,26 +994,28 @@ class TreeviewFilter(object):
         """
         Return a list of types in database.
         """
-        types = []
         fonts = Table('Fonts')
-        for row in set(fonts.get('filetype')):
-            types.append(row[0])
+        types = [row[0] for row in set(fonts.get('filetype'))]
         fonts.close()
         return natural_sort(types)
 
     def _setup_combos(self):
         model1 = gtk.ListStore(gobject.TYPE_STRING)
         model2 = gtk.ListStore(gobject.TYPE_STRING)
-        for entry in 'contains', 'begins with', 'ends with':
+        model3 = gtk.ListStore(gobject.TYPE_STRING)
+        for entry in _('contains'), _('begins with'), _('ends with'):
             model1.append([entry])
         for entry in '=', '!=':
             model2.append([entry])
+        for entry in self._family_types:
+            model3.append([entry])
         self.widgets['FamilyCombo'].set_model(model1)
-        self.widgets['TypeCombo'].set_model(model2)
+        self.widgets['FiletypeCombo'].set_model(model2)
         self.widgets['FoundryCombo'].set_model(model2)
         self.widgets['FilepathCombo'].set_model(model1)
+        self.widgets['TypeCombo'].set_model(model3)
         cell = gtk.CellRendererText()
-        for widget in self.combos:
+        for widget in self._combos:
             self.widgets[widget].pack_start(cell, True)
             self.widgets[widget].add_attribute(cell, 'text', 0)
             self.widgets[widget].set_active(0)
@@ -956,9 +1028,9 @@ class TreeviewFilter(object):
             model1.append([entry])
         for entry in self._foundries:
             model2.append([entry])
-        self.widgets['TypeComboEntry'].set_model(model1)
+        self.widgets['FiletypeComboEntry'].set_model(model1)
         self.widgets['FoundryComboEntry'].set_model(model2)
-        for widget in 'TypeComboEntry', 'FoundryComboEntry':
+        for widget in 'FiletypeComboEntry', 'FoundryComboEntry':
             self.widgets[widget].set_text_column(0)
         return
 
@@ -966,16 +1038,13 @@ class TreeviewFilter(object):
         """
         Display "advanced search" dialog.
         """
-        for widget in self.combos:
+        for widget in self._combos:
             self.widgets[widget].set_active(0)
         for widget in 'FamilyEntry', 'FilepathEntry':
             self.widgets[widget].set_text('')
-        for widget in 'TypeComboEntry', 'FoundryComboEntry':
+        for widget in 'FiletypeComboEntry', 'FoundryComboEntry':
             self.widgets[widget].child.set_text('')
-        dialog = self.widgets['SearchDialog']
-        response = dialog.run()
-        dialog.hide()
-        return response
+        return run_dialog(dialog = self.widgets['SearchDialog'])
 
     def run(self):
         if self._run_dialog():
@@ -986,13 +1055,4 @@ class TreeviewFilter(object):
 
 def get_header(title):
     header = glib.markup_escape_text(title)
-    return '<span size="xx-large" weight="heavy">%s</span>' % header
-
-# this is strictly cosmetic
-# Todo: make this nice instead of just a stock icon
-def _begin_drag(widget, context):
-    """
-    Set custom drag icon.
-    """
-    context.set_icon_name('font-x-generic', 24, 24)
-    return
+    return '<span size="xx-large" weight="heavy">{0}</span>'.format(header)
