@@ -40,8 +40,8 @@ from constants import USER_FONT_DIR, USER_FONT_CONFIG_DIR, \
                         USER_FONT_CONFIG_DESELECT, USER_FONT_COLLECTIONS, \
                         USER_FONT_COLLECTIONS_BAK, USER_ACTIONS_CONFIG, \
                         COMPAT_COLLECTIONS
-from constants import CHECKBUTTONS, CONSTS, CONSTS_MAP, DEFAULTS, SKIP, \
-                                            SLANT, WEIGHT, FC_WIDGETMAP, WIDTH
+from constants import FC_CHECKBUTTONS, FC_CONSTS, FC_CONSTS_MAP, FC_DEFAULTS, \
+                        FC_SKIP, PSLANT, PWEIGHT, FC_WIDGETMAP, PWIDTH
 
 def add_patelt_node(node_ref, node_value, pat_name='family', val_type='string'):
     """
@@ -79,7 +79,7 @@ def check_libxml2_leak():
     libxml2.cleanupParser()
     leak = libxml2.debugMemory(1)
     if leak > 0:
-        logging.debug("libxml2 --> memory leak %s bytes" % (leak))
+        logging.debug("libxml2 --> memory leak {0} bytes".format(leak))
     return
 
 def get_blacklisted():
@@ -304,7 +304,7 @@ def load_collections(fontmanager):
         comment = node.prop("comment")
         _get_fc_families(node, families)
         fontmanager.create_collection(name, comment, list(set(families)))
-        logging.info('Loaded user collection %s' % name)
+        logging.info('Loaded user collection {0}'.format(name))
         order.append(name)
     doc.freeDoc()
     load_compat_collections(fontmanager, order)
@@ -313,7 +313,7 @@ def load_collections(fontmanager):
 def load_compat_collections(fontmanager, order):
     """
     If ~/.config/fontgroups.xml exists load any collections defined there.
-    
+
     For compatibility with KDE font manager and any other programs which might
     make use of that file.
     """
@@ -329,13 +329,13 @@ def load_compat_collections(fontmanager, order):
         name = node.prop("name")
         if name in order:
             continue
-        comment = _('Created on %s' % \
+        comment = _('Created on {0}').format(
                 time.strftime('%A, %B %d, %Y, at %I:%M %p', time.localtime()))
         families = []
         for family in node.xpathEval('family'):
             families.append(family.content)
         fontmanager.create_collection(name, comment, list(set(families)))
-        logging.info('Imported user collection %s' % name)
+        logging.info('Imported user collection {0}'.format(name))
         order.append(name)
     doc.freeDoc()
     return
@@ -384,12 +384,11 @@ def save_collections(objects):
 def save_compat_collections(objects):
     """
     Save user defined collections to ~/.config/fontgroups.xml.
-    
+
     For compatibility with KDE font manager and any other programs which might
     make use of that file.
     """
-    if exists(COMPAT_COLLECTIONS):
-        os.unlink(COMPAT_COLLECTIONS)
+    not exists(COMPAT_COLLECTIONS) or os.unlink(COMPAT_COLLECTIONS)
     order = _get_collection_order(objects)
     printed = []
     # Start "printing"
@@ -397,7 +396,7 @@ def save_compat_collections(objects):
     root = doc.newChild(None, "groups", None)
     try:
         while len(order) != 0:
-            name = order[0]
+            name = order.pop(0)
             if name not in printed:
                 collection = objects['FontManager'].collections[name]
                 node = root.newChild(None, "group", None)
@@ -405,7 +404,6 @@ def save_compat_collections(objects):
                 for family in collection.families:
                     node.newChild(None, 'family', family)
                 printed.append(name)
-            order.pop(0)
     except:
         doc.freeDoc()
         return
@@ -421,25 +419,25 @@ def save_fontconfig_settings(settings):
     root = doc.newChild(None, 'fontconfig', None)
     for style in sorted(settings.faces.iterkeys()):
         dirty = False
-        for setting in CHECKBUTTONS:
+        for setting in FC_CHECKBUTTONS.keys():
             attribute = FC_WIDGETMAP[setting]
-            default_val = DEFAULTS[FC_WIDGETMAP[setting]]
+            default_val = FC_DEFAULTS[FC_WIDGETMAP[setting]]
             if getattr(settings.faces[style], attribute, default_val):
                 dirty = True
                 break
         if not dirty:
             continue
         node = _add_standard_match_targets(doc, root, style, settings)
-        less_eq = FC_WIDGETMAP[_('Smaller than')]
-        more_eq = FC_WIDGETMAP[_('Larger than')]
-        less = getattr(settings.faces[style], less_eq, DEFAULTS[less_eq])
-        more = getattr(settings.faces[style], more_eq, DEFAULTS[more_eq])
+        less_eq = FC_WIDGETMAP['Smaller than']
+        more_eq = FC_WIDGETMAP['Larger than']
+        less = getattr(settings.faces[style], less_eq, FC_DEFAULTS[less_eq])
+        more = getattr(settings.faces[style], more_eq, FC_DEFAULTS[more_eq])
         if less:
             test = node.newChild(None, 'test', None)
             test.setProp('name', 'size')
             test.setProp('compare', 'less')
             attribute = 'min_size'
-            default_val = DEFAULTS[attribute]
+            default_val = FC_DEFAULTS[attribute]
             val = int(getattr(settings.faces[style], attribute, default_val))
             test.newChild(None, 'double', str(val))
             _add_assignments(node, style, settings)
@@ -450,21 +448,21 @@ def save_fontconfig_settings(settings):
             test.setProp('name', 'size')
             test.setProp('compare', 'more')
             attribute = 'max_size'
-            default_val = DEFAULTS[attribute]
+            default_val = FC_DEFAULTS[attribute]
             val = int(getattr(settings.faces[style], attribute, default_val))
             test.newChild(None, 'double', str(val))
             _add_assignments(node, style, settings)
         if not less and not more:
             _add_assignments(node, style, settings)
-    doc.saveFormatFile(join(USER_FONT_CONFIG_DIR, 
-                        '25-%s.conf' % settings.family.get_name()), format=1)
+    doc.saveFormatFile(join(USER_FONT_CONFIG_DIR,
+                    '25-{0}.conf'.format(settings.family.get_name())), format=1)
     doc.freeDoc()
     return
 
 def _add_standard_match_targets(doc, root, style, settings):
     escape = glib.markup_escape_text
     comment = (settings.family.get_name(), style)
-    root.addChild(doc.newDocComment(' %s %s ' % comment))
+    root.addChild(doc.newDocComment(' {0} {1} '.format(*comment)))
     node = root.newChild(None, 'match', None)
     node.setProp('target', 'font')
     test = node.newChild(None, 'test', None)
@@ -478,40 +476,40 @@ def _guess_style_values(node, settings, style):
     test = node.newChild(None, 'test', None)
     test.setProp('name', 'slant')
     test.setProp('compare', 'eq')
-    test.newChild(None, 'const', SLANT[descr.get_style()])
+    test.newChild(None, 'const', PSLANT[descr.get_style()])
     test = node.newChild(None, 'test', None)
     test.setProp('name', 'weight')
     test.setProp('compare', 'more_eq')
-    test.newChild(None, 'int', WEIGHT[descr.get_weight()].split(':')[0])
+    test.newChild(None, 'int', PWEIGHT[descr.get_weight()].split(':')[0])
     test = node.newChild(None, 'test', None)
     test.setProp('name', 'weight')
     test.setProp('compare', 'less_eq')
-    test.newChild(None, 'int', WEIGHT[descr.get_weight()].split(':')[1])
+    test.newChild(None, 'int', PWEIGHT[descr.get_weight()].split(':')[1])
     test = node.newChild(None, 'test', None)
     test.setProp('name', 'width')
     test.setProp('compare', 'eq')
-    test.newChild(None, 'const', WIDTH[descr.get_stretch()])
+    test.newChild(None, 'const', PWIDTH[descr.get_stretch()])
     return
 
 def _add_assignments(node, style, settings):
-    for setting in CHECKBUTTONS:
+    for setting in FC_CHECKBUTTONS.keys():
         attribute = FC_WIDGETMAP[setting]
-        default_val = DEFAULTS[FC_WIDGETMAP[setting]]
+        default_val = FC_DEFAULTS[FC_WIDGETMAP[setting]]
         val = getattr(settings.faces[style], attribute, default_val)
-        if setting not in SKIP:
+        if setting not in FC_SKIP:
             edit = node.newChild(None, 'edit', None)
             edit.setProp('name', attribute)
             edit.setProp('mode', 'assign')
             edit.newChild(None, 'bool', str(val))
-        if val and attribute in CONSTS_MAP:
-            attribute = CONSTS_MAP[attribute]
-            default_val = DEFAULTS[attribute]
+        if val and attribute in FC_CONSTS_MAP:
+            attribute = FC_CONSTS_MAP[attribute]
+            default_val = FC_DEFAULTS[attribute]
             raw_val = getattr(settings.faces[style], attribute, default_val)
             val = str(raw_val).split('.')[1][:1]
             edit = node.newChild(None, 'edit', None)
             edit.setProp('name', attribute)
             edit.setProp('mode', 'assign')
-            edit.newChild(None, 'const', CONSTS[attribute][val])
+            edit.newChild(None, 'const', FC_CONSTS[attribute][val])
         if val and attribute == 'disablergba':
             edit = node.newChild(None, 'edit', None)
             edit.setProp('name', 'rgba')
@@ -538,7 +536,7 @@ def _tobool(val):
     """
     Convert from string to boolean.
     """
-    return val != 'False'
+    return (val != 'False' and val != 'false' and val != '0')
 
 def _unescape_markup(val):
     """
